// floating point function speed test - run on Rabbit at 14.7 MHz outputs performance for 29.4 MHz Rabbit
// this version runs on the Rabbit
// Similar version runs on Z180 under Dynamic C
// May be adapted to other processors.
//#define MS_TIMER 0  // uncomment for other C compilers
// remove all "nodebug" keywords for other compilers

/* ***** sample printouts **********                     
                                                                                
Times output are are for a 29.412 MHz Rabbit 2000 processor                     
                                                                                
!!!begin seive program n iterations                                             
###End seive program n iterations                                               
       0.090 seconds per iteration                                              
                                                                                
!!! begin empty loop                                                            
###end of empty loop                                                            
        4.52 microseconds per iteration                                         
                                                                                
!!! begin empty loop 2                                                          
###end of empty loop                                                            
        3.64 microseconds per iteration                                         
                                                                                
!!! begin empty loop 3 totally empty loop signed counter                        
###end of empty loop                                                            
        1.47 microseconds per iteration                                         
                                                                                
!!! begin empty loop 4 totally empty loop unsigned counter                      
###end of empty loop                                                            
        1.34 microseconds per iteration                                         
                                                                                
!!! begin if signed compare variable and constant                               
###end of  if signed compare 1                                                  
        1.27 microseconds per iteration                                         
                                                                                
!!! begin if unsigned compare variable and constant                             
###end of  if signed compare 1                                                  
        1.20 microseconds per iteration                                         
                                                                                
!!! begin floating multiply                                                     
###end of floating multiply                                                     
       12.10 microseconds per iteration                                         
                                                                                
!!! begin floating typical add                                                  
###end of floating add                                                          
        9.59 microseconds per iteration                                         
                                                                                
!!! begin floating #1 [1-1.000001] add                                          
###end of floating add                                                          
        9.40 microseconds per iteration                                         
                                                                                
!!! begin floating #2 [1+.000001] add                                           
###end of floating add                                                          
       10.16 microseconds per iteration                                         
                                                                                
!!! begin floating divide                                                       
###end of floating divide                                                       
       26.73 microseconds per iteration                                         
                                                                                
!!! begin floating square root                                                  
###end of floating square root                                                  
       31.55 microseconds per iteration                                         
                                                                                
!!! begin floating sine                                                         
###end of floating sine                                                         
       93.61 microseconds per iteration                                         
                                                                                
!!! begin floating exp                                                          
###end of floating exp                                                          
       93.45 microseconds per iteration                                         
                                                                                
!!! begin floating log                                                          
###end of floating log                                                          
      133.54 microseconds per iteration                                         
                                                                                
!!! begin floating atan                                                         
###end of floating atan                                                         
      118.67 microseconds per iteration                                        
                                         
*/



main(){

auto int i,j,k;
auto float x,y,z,time,emptytime,emptytime2,emptytime3,emptytime4;
auto unsigned long timer1,timer2;

auto int n;
float cf=.475; // clock factor, output times for 29.412 MHz processor using 14.73 MHz processor
             // also compensates for 5% time used by periodic interrupt


/*
This program must run on a 14.73 MHz processor or cf above must be modified.
Times output are for 14.73 MHz processor.
On 14.73 MHz processor clock interrupt slows processing by 5.0 % and cf takes this into account.

*/
printf("Times output are are for a 29.412 MHz Rabbit 2000 processor \n\n");


 

 // time the seive program, a test of integer operations
 
 n=2;
 printf("!!!begin seive program n iterations\n");
 timer1=MS_TIMER;
 seive(n);
 time=(MS_TIMER-timer1);
 time=cf*time/(n*1000.);
 printf("###End seive program n iterations\n");
 printf("%12.3f seconds per iteration\n\n",time);
 // .36 seconds per iteration on 14.7 MHz Rabbit
 
 
 // *** n must be multiple of 1000 in following tests
 
 n=100;
 printf("!!! begin empty loop\n");
 timer1=MS_TIMER;
 emptyloop(1.,n);
 time=(MS_TIMER-timer1);
 
 emptytime=cf*(time=1000.*(time/(n*1000l)));
 printf("###end of empty loop\n");
 printf("%12.2f microseconds per iteration\n\n",emptytime);
 
 
  n=100;
 printf("!!! begin empty loop 2\n");
 timer1=MS_TIMER;
 emptyloop2(1.,n);
 time=(MS_TIMER-timer1);
 
 emptytime2=cf*(time=1000.*(time/(n*1000l)));
 printf("###end of empty loop\n");
 printf("%12.2f microseconds per iteration\n\n",emptytime2);
 
  n=100;
 printf("!!! begin empty loop 3 totally empty loop signed counter\n");
 timer1=MS_TIMER;
 emptyloop3(1.,n);
 time=(MS_TIMER-timer1);
 
 emptytime3=cf*(time=1000.*(time/(n*1000l)));
 printf("###end of empty loop\n");
 printf("%12.2f microseconds per iteration\n\n",emptytime3);

 
  n=100;
 printf("!!! begin empty loop 4 totally empty loop unsigned counter\n");
 timer1=MS_TIMER;
 emptyloop4(1.,n);
 time=(MS_TIMER-timer1);
 
 emptytime4=cf*(time=1000.*(time/(n*1000l)));
 printf("###end of empty loop\n");
 printf("%12.2f microseconds per iteration\n\n",emptytime4);
 
  n=100;
 printf("!!! begin if signed compare variable and constant\n");
 timer1=MS_TIMER;
 ifloop1(n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000L)) - emptytime3;
 printf("###end of  if signed compare 1\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 
 n=100;
 printf("!!! begin if unsigned compare variable and constant\n");
 timer1=MS_TIMER;
 ifloop1u(n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000L)) - emptytime4;
 printf("###end of  if signed compare 1\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 
 n=10;
 printf("!!! begin floating multiply\n");
 timer1=MS_TIMER;
 floatingmul(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating multiply\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 


 n=10;
 printf("!!! begin floating typical add\n");
 timer1=MS_TIMER;
 floatingadd(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating add\n");
 printf("%12.2f microseconds per iteration\n\n",time);

  
 n=10;
 printf("!!! begin floating #1 [1-1.000001] add\n");
 timer1=MS_TIMER;
 floatingadd1(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating add\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 
 
 n=10;
 printf("!!! begin floating #2 [1+.000001] add\n");
 timer1=MS_TIMER;
 floatingadd2(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating add\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 
 
 
    n=10;
 printf("!!! begin floating divide\n");
 timer1=MS_TIMER;
 floatingdiv(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating divide\n");
 printf("%12.2f microseconds per iteration\n\n",time);

 
 
 
  n=10;
 printf("!!! begin floating square root\n");
 timer1=MS_TIMER;
 floatingsqrt(7.6834,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime2;
 printf("###end of floating square root\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 // 11.8 microseconds per iteration on 14.7 MHz Rabbit
 
 
 n=10;
 printf("!!! begin floating sine \n");
 timer1=MS_TIMER;
 floatingsin(.3,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime;
 printf("###end of floating sine\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 // 11.8 microseconds per iteration on 14.7 MHz Rabbit
 
 

 
 n=10;
 printf("!!! begin floating exp \n");
 timer1=MS_TIMER;
 floatingexp(.3,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime2;
 printf("###end of floating exp\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 // 11.8 microseconds per iteration on 14.7 MHz Rabbit
 
 
 n=10;
 printf("!!! begin floating log \n");
 timer1=MS_TIMER;
 floatinglog(.3,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime2;
 printf("###end of floating log\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 // 11.8 microseconds per iteration on 14.7 MHz Rabbit
 
 
 n=10;
 printf("!!! begin floating atan \n");
 timer1=MS_TIMER;
 floatingatan(.3,n);
 time=(MS_TIMER-timer1);
 time=cf*1000.*(time/(n*1000l)) - emptytime2;
 printf("###end of floating atan\n");
 printf("%12.2f microseconds per iteration\n\n",time);
 // 11.8 microseconds per iteration on 14.7 MHz Rabbit
 
 
 
 
  }
  
 float xxn[]={1.34897,5.67e-4,9234.564,8.564,456.33e5,1.0002,.0003467,1000.0034};
 float ttn[]={.01,.2, .4, .8, 1.002, -1.5324, 1.678, -.93}; // angles
 float ttl[]={ 1e20, 34., 1.02, 45., 22222., 1e30, 50000., 5666.} ; // numbers to log
 
nodebug  floatingsin(float x, int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7];
   for (k=0; k<1000; k++){
 
     y=sin(x); // null operation
   
   }
 }
} // return

nodebug  floatingexp(float x,  int n){
 auto int i,k,j;
 float y,z;
 

 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7];
   for (k=0; k<1000; k++){
 
     y=exp(x); // null operation
   
   }
 }
} // return

nodebug floatinglog(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttl[i&7];
   
   for (k=0; k<1000; k++){
 
     y=log(x); // null operation
   
   }
 }
} // return

 
nodebug floatingatan(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7]; // angles
   for (k=0; k<1000; k++){
 
     y=atan(x); // null operation
   
   }
 }
} // return



nodebug  floatingsqrt(float x, int n){
 auto int i,k,j;
 float y,z;
 
 
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=xxn[i&7];
   for (k=0; k<1000; k++){
 
     y=sqrt(x); // add operation
   
   } // k
    
 } // i
} // return
  






 // use this empty loop for binary functions
 nodebug emptyloop(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000; k++){
 
     y=x; // null operation
     z;   // try to balance time of empty loop
          // to isolate time of operation y=z*x
   }
 }
} // return


// use this empty loop for unary functions
 nodebug emptyloop2(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000; k++){
 
     y=x; // null operation
        // try to balance time of empty loop
          // to isolate time of operation y=z*x
   }
 }
} // return
    

// use this empty loop for stand alone statements 
 nodebug emptyloop3(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000; k++){
 
    
   }
 }
} // return
   

// If loop compare variable to signed constant 
 nodebug ifloop1(  int n){
 auto int i,k,j;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000; k++){
 
    if(k>245) goto xx; 
    xx:
   }
 }
} // return


// If loop compare variable to unsigned constant 
 nodebug ifloop1u(  int n){
 auto int i,j;
 auto unsigned int k;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000U; k++){
 
    if(k>245u) goto xx; 
    xx:
   }
 }
} // return



// Totally empty loop unsigned integer index
 nodebug emptyloop4(float x,  int n){
 auto int i,j;
 auto unsigned int k;
 float y,z;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000u; k++){
 
    
   }
 }
} // return
     

nodebug    floatingadd1(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 for(i=0; i<n; i++) {
   // inner loop
   x=1.; y=-1.000001;
   
   for (k=0; k<1000; k++){
 
     z=x+y; // add operation
   
   } // k
   
 } // i
} // return
  

nodebug    floatingadd2(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 for(i=0; i<n; i++) {
   // inner loop
  y=1.; x=.000001;
   
   for (k=0; k<1000; k++){
 
     z=x+y; // add operation
   
   } // k
   
 } // i
} // return


nodebug    floatingadd(float x,  int n){
 auto int i,k,j;
 float y,z;
 
 for(i=0; i<n; i++) {
   // inner loop
   x=xxn[i&7]; y=xxn[(i+i+1)&7];
   
   for (k=0; k<1000; k++){
 
     z=x+y; // add operation
   
   } // k
   
 } // i
} // return
  
nodebug  floatingmul(float x,  int n){
 auto int i,k,j;
 float y,z;
  x=xxn[i&7]; y=xxn[(i+i+1)&7];
 
 for(i=0; i<n; i++) {
   // inner loop
  
   for (k=0; k<1000; k++){
 
     z=x*y; // mul operation
   
   }
 }
} // return



nodebug  floatingdiv(float x, int n){
 auto int i,k,j;
 float y,z;

  x=xxn[i&7]; y=xxn[(i+i+1)&7];
 for(i=0; i<n; i++) {
   // inner loop

   for (k=0; k<1000; k++){
 
     z=x/y; // divide operation
    
   } // k
 } // i
} // return



    
nodebug seive( int n)  /* compute prines using seive of Eratosthenes*/
/* seive test program*/


#define SIZE 8190
#define FALSE 0
#define TRUE 1
//#define NTIMES 50


{
auto int i,j,k,count,prime,NTIMES;
static char flag[SIZE+1];

//printf("100 iterations: ");
NTIMES=n;

for (i=1; i<=NTIMES; i++)
	{
	count=0;
	for (j=0; j<=SIZE; j++) flag[j]=TRUE;

	for (j=0; j<=SIZE; j++)
		{if(flag[j])
			{
			prime=2*j+3;
			for(k=j+prime; k<= SIZE; k+=prime)
				flag[k]=FALSE; /*discard multiples*/
			count++;
			}
		}
	}
//printf("%d primes.\n",count);

}



