/*
 * Benchmark code for Intel 8051
 *
 */

#include <reg51.h>
#include <math.h>
#include <stdio.h>

unsigned long results[10]={0,0,0,0,0,0,0,0,0,0};
unsigned int idx=0;

 void floatingatan(float x,int n);
 void seive(long int n);
 void floatingmul(float x, int n);
 void floatingadd(float x, int n);
 void emptyloop(float x, int n);
 void floatingadd(float x, int n);
 void floatingdiv(float x, int n);
 void floatingsqrt(float x, int n);
 void floatingsin(float x, int n);
 void floatingexp(float x, int n);
 void floatingatan(float x, int n);
 void floatinglog(float x, int n);


	/* 19200 */
#define BAUDRATE 0xee



static volatile unsigned char data hl0, hh0;

void sw_reset(void)
{

     TR0 = 0;
     TMOD &= 0XF0;                    /* clear Timer 0   */
     TMOD  |= 0X1;
     TL0 = 0;                   /* value set by user    */
     TH0 = 0;                  /* value set by user  */
     ET0 = 1;     
     hl0=hh0 = 0;
     TR0 = 0;

	/*
 	 IE=0X2;
	  IP=0X0;
	  IPH=0X0;
	*/
}


static void timer0_isr(void) interrupt 1 using 1
{
	hl0++;
	if(!hl0) hh0++;
}

#define SW_START TR0=1
#define SW_STOP TR0=0





putc(char c)
{
	SBUF = c;
	while (!TI);
	TI=0;
}


void com_init(void)
{	/*Timer 1 is being used to generate baud rates.*/
     SCON = 0X40;
			
     PCON = 0X80 | PCON;
     TMOD = 0X20;
     TH1 = 0XFD;
     TR1 = 1;
     SBUF = 0X0;
     TI = 0;
}



void done(void)
{
	putc( 'D' ) ;
	putc( 'o' ) ;
	putc( 'n' ) ;
	putc( 'e' ) ;
	putc( '.' ) ;
	putc( '\n' ) ;
	putc( '\r' ) ;
}

void start(void)
{
	putc( 'S' ) ;
	putc( 't' ) ;
	putc( 'a' ) ;
	putc( 'r' ) ;
	putc( 't' ) ;
	putc( '.' ) ;
	putc( '\n' ) ;
	putc( '\r' ) ;
}



#define CVT(a,b, ch)  { (a) =  (ch) >> 4; \
	  if ((a)<10) (a) = (a) + '0'; \
          else (a) = (a)+ 'A' - 10;  \
	  (b) = (ch) & 0xf;	\
	  if ((b)<10) (b) = (b) + '0'; \
          else (b) = (b)+ 'A' - 10; }

void sw_report(void)
{
	unsigned char obuf[40];
	unsigned int i;
	unsigned int j;
	unsigned long et;
	float es;

	et = hh0;
	et = (et << 8) + hl0;
	et = (et << 8) + TH0;
	et = (et << 8) + TL0;

	results[idx] = et;
	idx++;

	es = et / 25.0e+6;
	es = es * 12;

	i = sprintf( obuf, "%ld\n\r", et);

	// i = sprintf( obuf, "%f\n\r", es );

	for (j=0; j<i; j++)
	   putc( obuf[j] ) ;

}

main()
{

//auto int i,j,k;
//auto float x,y,z,time,emptytime;
//auto unsigned long timer1,timer2;
int n;
        idx=0;
	com_init();
	EA=1;
#if 0
	start();

	sw_reset();
	SW_START;

   // time the seive program, a test of integer operations

 n=400;

 ;
 seive(n);

	SW_STOP;
	sw_report();

// seive -  188 40 mhz - 12 sec .12 sec / iteration
#endif

  // *** n must be multiple of 1000 in following tests
 
 n=5000;
	sw_reset();
	SW_START;

 emptyloop(1.,n);
	SW_STOP;
	sw_report();

 // empty loop

#if 0

 n=100;
	sw_reset();
	SW_START;

 floatingmul(7.6834,n);
	SW_STOP;
	sw_report();

 // floating mul




 n=100;
	sw_reset();
	SW_START;

 floatingadd(7.6834,n);
	SW_STOP;
	sw_report();

 // floating add
#endif

    n=100;

	sw_reset();
	SW_START;
 floatingdiv(7.6834,n);
	SW_STOP;
	sw_report();


// floating div

  n=100;

	sw_reset();
	SW_START;
 floatingsqrt(7.6834,n);
	SW_STOP;
	sw_report();

// floating sqrt
 n=50;

	sw_reset();
	SW_START;
 floatingsin(.3,n);
	SW_STOP;
	sw_report();

// floating sin

 
 n=20;

	sw_reset();
	SW_START;
 floatingexp(.3,n);
	SW_STOP;
	sw_report();

 // floating exp


 n=20;

	sw_reset();
	SW_START;
 floatinglog(.3,n);
	SW_STOP;
	sw_report();

 // floating log


 n=20;

	sw_reset();
	SW_START;
 floatingatan(.3,n);
	SW_STOP;
	sw_report();

 // floating atan



	done();
  
}


 float xxn[]={1.34897,5.67e-4,9234.564,8.564,456.33e5,1.0002,.0003467,1000.0034};
 float ttn[]={.01,.2, .4, .8, 1.002, -1.5324, 1.678, -.93}; // angles
 float ttl[]={ 1e20, 34., 1.02, 45., 22222., 1e30, 50000., 5666.} ; // numbers to log
 
void floatingsin(float x, int n){
 auto int i,k;
 float y;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7];
   for (k=0; k<1000; k++){
 
     y=sin(x); // null operation
   
   }
 }
} // return

void floatingexp(float x,  int n){
 auto int i,k;
 float y;
 

 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7];
   for (k=0; k<1000; k++){
 
     y=exp(x); // null operation
   
   }
 }
} // return

void floatinglog(float x,  int n){
 auto int i,k;
 float y;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttl[i&7];
   
   for (k=0; k<1000; k++){
 
     y=log(x); // null operation
   
   }
 }
} // return

 
void floatingatan(float x,  int n){
 auto int i,k;
 float y;
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=ttn[i&7]; // angles
   for (k=0; k<1000; k++){
 
     y=atan(x); // null operation
   
   }
 }
} // return



void floatingsqrt(float x, int n){
 auto int i,k;
 float y;
 
 
 
 
 for(i=0; i<n; i++) {
   // inner loop
   x=xxn[i&7];
   for (k=0; k<1000; k++){
 
     y=sqrt(x); // add operation
   
   } // k
    
 } // i
} // return
  






void emptyloop(float x,  int n){
 auto int i,k;
 float y;
 
 for(i=0; i<n; i++) {
   // inner loop
   for (k=0; k<1000; k++){
 
     y=x; // null operation
   
   }
 }
} // return


 void  floatingadd(float x,  int n){
 auto int i,k;
 float y,z;
 
  
 
 for(i=0; i<n; i++) {
   // inner loop
   x=xxn[i&7]; y=xxn[(i+i+1)&7];
   
   for (k=0; k<1000; k++){
 
     z=x+y; // add operation
   
   } // k
   
 } // i
} // return
  
  
void floatingmul(float x,  int n){
 auto int i,k;
 float y,z;
  x=xxn[i&7]; y=xxn[(i+i+1)&7];
 
 for(i=0; i<n; i++) {
   // inner loop
  
   for (k=0; k<1000; k++){
 
     z=x*y; // mul operation
   
   }
 }
} // return


void  floatingdiv(float x, int n){
 auto int i,k;
 float y,z;

  x=xxn[i&7]; y=xxn[(i+i+1)&7];
 for(i=0; i<n; i++) {
   // inner loop

   for (k=0; k<1000; k++){
 
     z=x/y; // divide operation
    
   } // k
 } // i
} // return



    
void seive(long int n)  /* compute prines using seive of Eratosthenes*/
/* seive test program*/


#define SIZE 8190
#define FALSE 0
#define TRUE 1
//#define NTIMES 50


{
auto int i,k,j,count,prime,NTIMES;
static char flag[SIZE+1];

//printf("100 iterations: ");
NTIMES=n;

for (i=1; i<=NTIMES; i++)
	{
	count=0;
	for (j=0; j<=SIZE; j++) flag[j]=TRUE;
	for (j=0; j<=SIZE; j++)
		{if(flag[j])
			{
			prime=2*j+3;
			for(k=j+prime; k<= SIZE; k+=prime)
				flag[k]=FALSE; /*discard multiples*/
			count++;
			}
		}
	}
//printf("%d primes.\n",count);

}


